"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DidExchangeCompleteHandler = void 0;
const error_1 = require("../../../error");
const messages_1 = require("../messages");
class DidExchangeCompleteHandler {
    constructor(didExchangeProtocol, outOfBandService) {
        this.supportedMessages = [messages_1.DidExchangeCompleteMessage];
        this.didExchangeProtocol = didExchangeProtocol;
        this.outOfBandService = outOfBandService;
    }
    async handle(messageContext) {
        var _a, _b, _c;
        const { connection: connectionRecord } = messageContext;
        if (!connectionRecord) {
            throw new error_1.AriesFrameworkError(`Connection is missing in message context`);
        }
        const { protocol } = connectionRecord;
        if (protocol !== "https://didcomm.org/didexchange/1.0" /* DidExchange */) {
            throw new error_1.AriesFrameworkError(`Connection record protocol is ${protocol} but handler supports only ${"https://didcomm.org/didexchange/1.0" /* DidExchange */}.`);
        }
        const { message } = messageContext;
        if (!((_a = message.thread) === null || _a === void 0 ? void 0 : _a.parentThreadId)) {
            throw new error_1.AriesFrameworkError(`Message does not contain pthid attribute`);
        }
        const outOfBandRecord = await this.outOfBandService.findByInvitationId((_b = message.thread) === null || _b === void 0 ? void 0 : _b.parentThreadId);
        if (!outOfBandRecord) {
            throw new error_1.AriesFrameworkError(`OutOfBand record for message ID ${(_c = message.thread) === null || _c === void 0 ? void 0 : _c.parentThreadId} not found!`);
        }
        if (!outOfBandRecord.reusable) {
            await this.outOfBandService.updateState(outOfBandRecord, "done" /* Done */);
        }
        await this.didExchangeProtocol.processComplete(messageContext, outOfBandRecord);
    }
}
exports.DidExchangeCompleteHandler = DidExchangeCompleteHandler;
//# sourceMappingURL=DidExchangeCompleteHandler.js.map