"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.IndyCredentialFormatService = void 0;
const tsyringe_1 = require("tsyringe");
const AgentConfig_1 = require("../../../../agent/AgentConfig");
const EventEmitter_1 = require("../../../../agent/EventEmitter");
const error_1 = require("../../../../error");
const JsonTransformer_1 = require("../../../../utils/JsonTransformer");
const MessageValidator_1 = require("../../../../utils/MessageValidator");
const uuid_1 = require("../../../../utils/uuid");
const indy_1 = require("../../../indy");
const ledger_1 = require("../../../ledger");
const CredentialAutoAcceptType_1 = require("../../CredentialAutoAcceptType");
const CredentialUtils_1 = require("../../CredentialUtils");
const CredentialsModuleOptions_1 = require("../../CredentialsModuleOptions");
const composeAutoAccept_1 = require("../../composeAutoAccept");
const errors_1 = require("../../errors");
const CredentialPreviewAttribute_1 = require("../../models/CredentialPreviewAttribute");
const V2CredentialPreview_1 = require("../../protocol/v2/V2CredentialPreview");
const CredentialMetadataTypes_1 = require("../../repository/CredentialMetadataTypes");
const CredentialRepository_1 = require("../../repository/CredentialRepository");
const CredentialFormatService_1 = require("../CredentialFormatService");
const CredPropose_1 = require("../models/CredPropose");
const CredentialFormatServiceOptions_1 = require("../models/CredentialFormatServiceOptions");
let IndyCredentialFormatService = class IndyCredentialFormatService extends CredentialFormatService_1.CredentialFormatService {
    constructor(credentialRepository, eventEmitter, indyIssuerService, indyLedgerService, indyHolderService, agentConfig) {
        super(credentialRepository, eventEmitter);
        this.credentialRepository = credentialRepository;
        this.indyIssuerService = indyIssuerService;
        this.indyLedgerService = indyLedgerService;
        this.indyHolderService = indyHolderService;
        this.logger = agentConfig.logger;
    }
    /**
     * Create a {@link AttachmentFormats} object dependent on the message type.
     *
     * @param options The object containing all the options for the proposed credential
     * @param messageType the type of message which can be Indy, JsonLd etc eg "CRED_20_PROPOSAL"
     * @returns object containing associated attachment, formats and filtersAttach elements
     *
     */
    async createProposal(options) {
        var _a, _b;
        const formats = {
            attachId: this.generateId(),
            format: 'hlindy/cred-filter@v2.0',
        };
        if (!((_a = options.credentialFormats.indy) === null || _a === void 0 ? void 0 : _a.payload)) {
            throw new error_1.AriesFrameworkError('Missing payload in createProposal');
        }
        // Use class instance instead of interface, otherwise this causes interoperability problems
        let proposal = new CredPropose_1.CredPropose((_b = options.credentialFormats.indy) === null || _b === void 0 ? void 0 : _b.payload);
        try {
            await MessageValidator_1.MessageValidator.validate(proposal);
        }
        catch (error) {
            throw new error_1.AriesFrameworkError(`Invalid credPropose class instance: ${proposal} in Indy Format Service`);
        }
        proposal = JsonTransformer_1.JsonTransformer.toJSON(proposal);
        const attachment = this.getFormatData(proposal, formats.attachId);
        const { previewWithAttachments } = this.getCredentialLinkedAttachments(options);
        return { format: formats, attachment, preview: previewWithAttachments };
    }
    async processProposal(options, credentialRecord) {
        var _a;
        const credProposalJson = (_a = options.proposalAttachment) === null || _a === void 0 ? void 0 : _a.getDataAsJson();
        if (!credProposalJson) {
            throw new error_1.AriesFrameworkError('Missing indy credential proposal data payload');
        }
        const credProposal = JsonTransformer_1.JsonTransformer.fromJSON(credProposalJson, CredPropose_1.CredPropose);
        await MessageValidator_1.MessageValidator.validate(credProposal);
        if (credProposal.credentialDefinitionId) {
            options.credentialFormats = {
                indy: {
                    credentialDefinitionId: credProposal === null || credProposal === void 0 ? void 0 : credProposal.credentialDefinitionId,
                    attributes: [],
                },
            };
        }
        credentialRecord.metadata.set(CredentialMetadataTypes_1.CredentialMetadataKeys.IndyCredential, {
            schemaId: credProposal.schemaId,
            credentialDefinitionId: credProposal.credentialDefinitionId,
        });
    }
    /**
     * Create a {@link AttachmentFormats} object dependent on the message type.
     *
     * @param options The object containing all the options for the credential offer
     * @param messageType the type of message which can be Indy, JsonLd etc eg "CRED_20_OFFER"
     * @returns object containing associated attachment, formats and offersAttach elements
     *
     */
    async createOffer(options) {
        var _a, _b;
        const formats = new CredentialFormatServiceOptions_1.CredentialFormatSpec({
            attachId: this.generateId(),
            format: 'hlindy/cred-abstract@v2.0',
        });
        const offer = await this.createCredentialOffer(options);
        let preview;
        if ((_a = options === null || options === void 0 ? void 0 : options.credentialFormats.indy) === null || _a === void 0 ? void 0 : _a.attributes) {
            preview = new V2CredentialPreview_1.V2CredentialPreview({
                attributes: (_b = options === null || options === void 0 ? void 0 : options.credentialFormats.indy) === null || _b === void 0 ? void 0 : _b.attributes.map((attribute) => new CredentialPreviewAttribute_1.CredentialPreviewAttribute(attribute)),
            });
        }
        // if the proposal has an attachment Id use that, otherwise the generated id of the formats object
        const attachmentId = options.attachId ? options.attachId : formats.attachId;
        const offersAttach = this.getFormatData(offer, attachmentId);
        // with credential preview now being a required field (as per spec)
        // attributes could be empty
        if (preview && preview.attributes.length > 0) {
            await this.checkPreviewAttributesMatchSchemaAttributes(offersAttach, preview);
        }
        return { format: formats, attachment: offersAttach, preview };
    }
    async processOffer(attachment, credentialRecord) {
        if (!attachment) {
            throw new error_1.AriesFrameworkError('Missing offer attachment in processOffer');
        }
        this.logger.debug(`Save metadata for credential record ${credentialRecord.id}`);
        const credOffer = attachment.getDataAsJson();
        credentialRecord.metadata.set(CredentialMetadataTypes_1.CredentialMetadataKeys.IndyCredential, {
            schemaId: credOffer.schema_id,
            credentialDefinitionId: credOffer.cred_def_id,
        });
    }
    /**
     * Create a {@link AttachmentFormats} object dependent on the message type.
     *
     * @param requestOptions The object containing all the options for the credential request
     * @param credentialRecord the credential record containing the offer from which this request
     * is derived
     * @returns object containing associated attachment, formats and requestAttach elements
     *
     */
    async createRequest(options, credentialRecord) {
        var _a;
        if (!options.offerAttachment) {
            throw new error_1.AriesFrameworkError(`Missing attachment from offer message, credential record id = ${credentialRecord.id}`);
        }
        if (!options.holderDid) {
            throw new error_1.AriesFrameworkError(`Missing holder DID from offer message, credential record id = ${credentialRecord.id}`);
        }
        const offer = options.offerAttachment.getDataAsJson();
        const credDef = await this.getCredentialDefinition(offer);
        const { credReq, credReqMetadata } = await this.createIndyCredentialRequest(offer, credDef, options.holderDid);
        credentialRecord.metadata.set(CredentialMetadataTypes_1.CredentialMetadataKeys.IndyRequest, credReqMetadata);
        const formats = new CredentialFormatServiceOptions_1.CredentialFormatSpec({
            attachId: this.generateId(),
            format: 'hlindy/cred-req@v2.0',
        });
        const attachmentId = (_a = options.attachId) !== null && _a !== void 0 ? _a : formats.attachId;
        const requestAttach = this.getFormatData(credReq, attachmentId);
        return { format: formats, attachment: requestAttach };
    }
    /**
     * Not implemented; there for future versions
     */
    async processRequest(
    /* eslint-disable @typescript-eslint/no-unused-vars */
    _options, _credentialRecord
    /* eslint-enable @typescript-eslint/no-unused-vars */
    ) {
        // not needed for Indy
    }
    async getCredentialDefinition(credOffer) {
        const indyCredDef = await this.indyLedgerService.getCredentialDefinition(credOffer.cred_def_id);
        return indyCredDef;
    }
    /**
     * Get linked attachments for indy format from a proposal message. This allows attachments
     * to be copied across to old style credential records
     *
     * @param options ProposeCredentialOptions object containing (optionally) the linked attachments
     * @return array of linked attachments or undefined if none present
     */
    getCredentialLinkedAttachments(options) {
        var _a;
        // Add the linked attachments to the credentialProposal
        if (!((_a = options.credentialFormats.indy) === null || _a === void 0 ? void 0 : _a.payload)) {
            throw new error_1.AriesFrameworkError('Missing payload in getCredentialLinkedAttachments');
        }
        let attachments;
        let previewWithAttachments;
        if (options.credentialFormats.indy.attributes) {
            previewWithAttachments = new V2CredentialPreview_1.V2CredentialPreview({
                attributes: options.credentialFormats.indy.attributes.map((attribute) => new CredentialPreviewAttribute_1.CredentialPreviewAttribute(attribute)),
            });
        }
        if (!options.credentialFormats.indy.attributes) {
            throw new error_1.AriesFrameworkError('Missing attributes from credential proposal');
        }
        if (options.credentialFormats.indy && options.credentialFormats.indy.linkedAttachments) {
            // there are linked attachments so transform into the attribute field of the CredentialPreview object for
            // this proposal
            previewWithAttachments = CredentialUtils_1.CredentialUtils.createAndLinkAttachmentsToPreview(options.credentialFormats.indy.linkedAttachments, new V2CredentialPreview_1.V2CredentialPreview({
                attributes: options.credentialFormats.indy.attributes.map((attribute) => new CredentialPreviewAttribute_1.CredentialPreviewAttribute(attribute)),
            }));
            attachments = options.credentialFormats.indy.linkedAttachments.map((linkedAttachment) => linkedAttachment.attachment);
        }
        if (!previewWithAttachments) {
            throw new error_1.AriesFrameworkError('No previewWithAttachments');
        }
        return { attachments, previewWithAttachments };
    }
    /**
     * Gets the attachment object for a given attachId. We need to get out the correct attachId for
     * indy and then find the corresponding attachment (if there is one)
     * @param formats the formats object containing the attachid
     * @param messageAttachment the attachment containing the payload
     * @returns The Attachment if found or undefined
     */
    getAttachment(formats, messageAttachment) {
        const formatId = formats.find((f) => f.format.includes('indy'));
        const attachment = messageAttachment === null || messageAttachment === void 0 ? void 0 : messageAttachment.find((attachment) => attachment.id === (formatId === null || formatId === void 0 ? void 0 : formatId.attachId));
        return attachment;
    }
    /**
     * Create a credential offer for the given credential definition id.
     *
     * @param credentialDefinitionId The credential definition to create an offer for
     * @returns The created credential offer
     */
    async createCredentialOffer(proposal) {
        var _a, _b;
        if (!((_b = (_a = proposal.credentialFormats) === null || _a === void 0 ? void 0 : _a.indy) === null || _b === void 0 ? void 0 : _b.credentialDefinitionId)) {
            throw new error_1.AriesFrameworkError('Missing Credential Definition id');
        }
        const credOffer = await this.indyIssuerService.createCredentialOffer(proposal.credentialFormats.indy.credentialDefinitionId);
        return credOffer;
    }
    /**
     * Create a credential offer for the given credential definition id.
     *
     * @param options RequestCredentialOptions the config options for the credential request
     * @throws Error if unable to create the request
     * @returns The created credential offer
     */
    async createIndyCredentialRequest(offer, credentialDefinition, holderDid) {
        const [credReq, credReqMetadata] = await this.indyHolderService.createCredentialRequest({
            holderDid: holderDid,
            credentialOffer: offer,
            credentialDefinition,
        });
        return { credReq, credReqMetadata };
    }
    /**
     * Create a {@link AttachmentFormats} object dependent on the message type.
     *
     * @param requestOptions The object containing all the options for the credential request
     * @param credentialRecord the credential record containing the offer from which this request
     * is derived
     * @returns object containing associated attachment, formats and requestAttach elements
     *
     */
    async createCredential(options, record, requestAttachment, offerAttachment) {
        // Assert credential attributes
        const credentialAttributes = record.credentialAttributes;
        if (!credentialAttributes) {
            throw new errors_1.CredentialProblemReportError(`Missing required credential attribute values on credential record with id ${record.id}`, { problemCode: errors_1.CredentialProblemReportReason.IssuanceAbandoned });
        }
        const credOffer = offerAttachment === null || offerAttachment === void 0 ? void 0 : offerAttachment.getDataAsJson();
        const credRequest = requestAttachment === null || requestAttachment === void 0 ? void 0 : requestAttachment.getDataAsJson();
        if (!credOffer || !credRequest) {
            throw new error_1.AriesFrameworkError('Missing CredOffer or CredReq in createCredential');
        }
        if (!this.indyIssuerService) {
            throw new error_1.AriesFrameworkError('Missing indyIssuerService in createCredential');
        }
        const [credential] = await this.indyIssuerService.createCredential({
            credentialOffer: credOffer,
            credentialRequest: credRequest,
            credentialValues: CredentialUtils_1.CredentialUtils.convertAttributesToValues(credentialAttributes),
        });
        const formats = new CredentialFormatServiceOptions_1.CredentialFormatSpec({
            attachId: this.generateId(),
            format: 'hlindy/cred-abstract@v2.0',
        });
        const attachmentId = options.attachId ? options.attachId : formats.attachId;
        const issueAttachment = this.getFormatData(credential, attachmentId);
        return { format: formats, attachment: issueAttachment };
    }
    /**
     * Processes an incoming credential - retrieve metadata, retrieve payload and store it in the Indy wallet
     * @param message the issue credential message
     */
    /**
     * Processes an incoming credential - retrieve metadata, retrieve payload and store it in the Indy wallet
     * @param options the issue credential message wrapped inside this object
     * @param credentialRecord the credential exchange record for this credential
     */
    async processCredential(options, credentialRecord) {
        var _a;
        const credentialRequestMetadata = credentialRecord.metadata.get(CredentialMetadataTypes_1.CredentialMetadataKeys.IndyRequest);
        if (!credentialRequestMetadata) {
            throw new errors_1.CredentialProblemReportError(`Missing required request metadata for credential with id ${credentialRecord.id}`, { problemCode: errors_1.CredentialProblemReportReason.IssuanceAbandoned });
        }
        if (!options.credentialAttachment) {
            throw new error_1.AriesFrameworkError(`Missing credential for record id ${credentialRecord.id}`);
        }
        const indyCredential = options.credentialAttachment.getDataAsJson();
        const credentialDefinition = await this.indyLedgerService.getCredentialDefinition(indyCredential.cred_def_id);
        if (!options.credentialAttachment) {
            throw new error_1.AriesFrameworkError('Missing credential attachment in processCredential');
        }
        const revocationRegistry = await this.getRevocationRegistry(options.credentialAttachment);
        const credentialId = await this.indyHolderService.storeCredential({
            credentialId: this.generateId(),
            credentialRequestMetadata,
            credential: indyCredential,
            credentialDefinition,
            revocationRegistryDefinition: (_a = revocationRegistry === null || revocationRegistry === void 0 ? void 0 : revocationRegistry.indy) === null || _a === void 0 ? void 0 : _a.revocationRegistryDefinition,
        });
        credentialRecord.credentials.push({
            credentialRecordType: CredentialsModuleOptions_1.CredentialFormatType.Indy,
            credentialRecordId: credentialId,
        });
    }
    /**
     * Checks whether it should automatically respond to a proposal. Moved from CredentialResponseCoordinator
     * as this contains format-specific logic
     * @param credentialRecord The credential record for which we are testing whether or not to auto respond
     * @param agentConfig config object for the agent, used to hold auto accept state for the agent
     * @returns true if we should auto respond, false otherwise
     */
    shouldAutoRespondToProposal(handlerOptions) {
        const autoAccept = composeAutoAccept_1.composeAutoAccept(handlerOptions.credentialRecord.autoAcceptCredential, handlerOptions.autoAcceptType);
        if (autoAccept === CredentialAutoAcceptType_1.AutoAcceptCredential.ContentApproved) {
            return (this.areProposalValuesValid(handlerOptions.credentialRecord, handlerOptions.messageAttributes) &&
                this.areProposalAndOfferDefinitionIdEqual(handlerOptions.proposalAttachment, handlerOptions.offerAttachment));
        }
        return false;
    }
    /**
   * Checks whether it should automatically respond to a request. Moved from CredentialResponseCoordinator
   * as this contains format-specific logic
   * @param credentialRecord The credential record for which we are testing whether or not to auto respond
   * @param autoAcceptType auto accept type for this credential exchange - normal auto or content approved
   * @returns true if we should auto respond, false otherwise
  
   */
    shouldAutoRespondToRequest(options) {
        const autoAccept = composeAutoAccept_1.composeAutoAccept(options.credentialRecord.autoAcceptCredential, options.autoAcceptType);
        if (!options.requestAttachment) {
            throw new error_1.AriesFrameworkError(`Missing Request Attachment for Credential Record ${options.credentialRecord.id}`);
        }
        if (autoAccept === CredentialAutoAcceptType_1.AutoAcceptCredential.ContentApproved) {
            return this.isRequestDefinitionIdValid(options.requestAttachment, options.offerAttachment, options.proposalAttachment);
        }
        return false;
    }
    /**
     * Checks whether it should automatically respond to a request. Moved from CredentialResponseCoordinator
     * as this contains format-specific logic
     * @param credentialRecord The credential record for which we are testing whether or not to auto respond
     * @param autoAcceptType auto accept type for this credential exchange - normal auto or content approved
     * @returns true if we should auto respond, false otherwise
     */
    shouldAutoRespondToCredential(options) {
        const autoAccept = composeAutoAccept_1.composeAutoAccept(options.credentialRecord.autoAcceptCredential, options.autoAcceptType);
        if (autoAccept === CredentialAutoAcceptType_1.AutoAcceptCredential.ContentApproved) {
            if (options.credentialAttachment) {
                return this.areCredentialValuesValid(options.credentialRecord, options.credentialAttachment);
            }
        }
        return false;
    }
    areProposalValuesValid(credentialRecord, proposeMessageAttributes) {
        const { credentialAttributes } = credentialRecord;
        if (proposeMessageAttributes && credentialAttributes) {
            const proposeValues = CredentialUtils_1.CredentialUtils.convertAttributesToValues(proposeMessageAttributes);
            const defaultValues = CredentialUtils_1.CredentialUtils.convertAttributesToValues(credentialAttributes);
            if (CredentialUtils_1.CredentialUtils.checkValuesMatch(proposeValues, defaultValues)) {
                return true;
            }
        }
        return false;
    }
    areProposalAndOfferDefinitionIdEqual(proposalAttachment, offerAttachment) {
        const credOffer = offerAttachment === null || offerAttachment === void 0 ? void 0 : offerAttachment.getDataAsJson();
        let credPropose = proposalAttachment === null || proposalAttachment === void 0 ? void 0 : proposalAttachment.getDataAsJson();
        credPropose = JsonTransformer_1.JsonTransformer.fromJSON(credPropose, CredPropose_1.CredPropose);
        const proposalCredentialDefinitionId = credPropose === null || credPropose === void 0 ? void 0 : credPropose.credentialDefinitionId;
        const offerCredentialDefinitionId = credOffer === null || credOffer === void 0 ? void 0 : credOffer.cred_def_id;
        return proposalCredentialDefinitionId === offerCredentialDefinitionId;
    }
    areCredentialValuesValid(credentialRecord, credentialAttachment) {
        const indyCredential = credentialAttachment.getDataAsJson();
        if (!indyCredential) {
            new error_1.AriesFrameworkError(`Missing required base64 encoded attachment data for credential`);
            return false;
        }
        const credentialMessageValues = indyCredential.values;
        if (credentialRecord.credentialAttributes) {
            const defaultValues = CredentialUtils_1.CredentialUtils.convertAttributesToValues(credentialRecord.credentialAttributes);
            if (CredentialUtils_1.CredentialUtils.checkValuesMatch(credentialMessageValues, defaultValues)) {
                return true;
            }
        }
        return false;
    }
    async deleteCredentialById(credentialRecordId) {
        await this.indyHolderService.deleteCredential(credentialRecordId);
    }
    async checkPreviewAttributesMatchSchemaAttributes(offerAttachment, preview) {
        const credOffer = offerAttachment === null || offerAttachment === void 0 ? void 0 : offerAttachment.getDataAsJson();
        const schema = await this.indyLedgerService.getSchema(credOffer.schema_id);
        CredentialUtils_1.CredentialUtils.checkAttributesMatch(schema, preview);
    }
    isRequestDefinitionIdValid(requestAttachment, offerAttachment, proposeAttachment) {
        var _a;
        const indyCredentialRequest = requestAttachment === null || requestAttachment === void 0 ? void 0 : requestAttachment.getDataAsJson();
        let indyCredentialProposal = proposeAttachment === null || proposeAttachment === void 0 ? void 0 : proposeAttachment.getDataAsJson();
        indyCredentialProposal = JsonTransformer_1.JsonTransformer.fromJSON(indyCredentialProposal, CredPropose_1.CredPropose);
        const indyCredentialOffer = offerAttachment === null || offerAttachment === void 0 ? void 0 : offerAttachment.getDataAsJson();
        if (indyCredentialProposal || indyCredentialOffer) {
            const previousCredentialDefinitionId = (_a = indyCredentialOffer === null || indyCredentialOffer === void 0 ? void 0 : indyCredentialOffer.cred_def_id) !== null && _a !== void 0 ? _a : indyCredentialProposal === null || indyCredentialProposal === void 0 ? void 0 : indyCredentialProposal.credentialDefinitionId;
            if (previousCredentialDefinitionId === indyCredentialRequest.cred_def_id) {
                return true;
            }
            return false;
        }
        return false;
    }
    generateId() {
        return uuid_1.uuid();
    }
    async getRevocationRegistry(issueAttachment) {
        const credential = issueAttachment.getDataAsJson();
        let indyRegistry;
        if (credential.rev_reg_id) {
            indyRegistry = await this.indyLedgerService.getRevocationRegistryDefinition(credential.rev_reg_id);
        }
        return { indy: indyRegistry };
    }
};
IndyCredentialFormatService = __decorate([
    tsyringe_1.scoped(tsyringe_1.Lifecycle.ContainerScoped),
    __metadata("design:paramtypes", [CredentialRepository_1.CredentialRepository,
        EventEmitter_1.EventEmitter,
        indy_1.IndyIssuerService,
        ledger_1.IndyLedgerService,
        indy_1.IndyHolderService,
        AgentConfig_1.AgentConfig])
], IndyCredentialFormatService);
exports.IndyCredentialFormatService = IndyCredentialFormatService;
//# sourceMappingURL=IndyCredentialFormatService.js.map