"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.V1IssueCredentialHandler = void 0;
const helpers_1 = require("../../../../../agent/helpers");
const AriesFrameworkError_1 = require("../../../../../error/AriesFrameworkError");
const messages_1 = require("../messages");
class V1IssueCredentialHandler {
    constructor(credentialService, agentConfig, didCommMessageRepository) {
        this.supportedMessages = [messages_1.V1IssueCredentialMessage];
        this.credentialService = credentialService;
        this.agentConfig = agentConfig;
        this.didCommMessageRepository = didCommMessageRepository;
    }
    async handle(messageContext) {
        const credentialRecord = await this.credentialService.processCredential(messageContext);
        const credentialMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: messages_1.V1IssueCredentialMessage,
        });
        if (!credentialMessage) {
            throw new AriesFrameworkError_1.AriesFrameworkError('Missing credential message in V2RequestCredentialHandler');
        }
        if (this.credentialService.shouldAutoRespondToCredential(credentialRecord, credentialMessage)) {
            return await this.createAck(credentialRecord, credentialMessage, messageContext);
        }
    }
    async createAck(record, credentialMessage, messageContext) {
        this.agentConfig.logger.info(`Automatically sending acknowledgement with autoAccept on ${this.agentConfig.autoAcceptCredentials}`);
        const { message, credentialRecord } = await this.credentialService.createAck(record);
        const requestMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: messages_1.V1RequestCredentialMessage,
        });
        if (messageContext.connection) {
            return helpers_1.createOutboundMessage(messageContext.connection, message);
        }
        else if ((credentialMessage === null || credentialMessage === void 0 ? void 0 : credentialMessage.service) && (requestMessage === null || requestMessage === void 0 ? void 0 : requestMessage.service)) {
            const recipientService = credentialMessage.service;
            const ourService = requestMessage.service;
            return helpers_1.createOutboundServiceMessage({
                payload: message,
                service: recipientService.resolvedDidCommService,
                senderKey: ourService.resolvedDidCommService.recipientKeys[0],
            });
        }
        this.agentConfig.logger.error(`Could not automatically create credential ack`);
    }
}
exports.V1IssueCredentialHandler = V1IssueCredentialHandler;
//# sourceMappingURL=V1IssueCredentialHandler.js.map